#!/bin/sh
# vim600: set foldmethod=marker: # Faster folding in VIM.
#
# Shell script to test charcode. {{{1
# This is intended to be a generic shell testing framework.
#
# Copyright (C) 2007 Lorance Stinson All rights reserved.  This script is free
# software.  It may be copied or modified according to the same terms as Perl.
#
# For more information email <LoranceStinson+charcode@gmail.com>.
# Or see http://lorance.freeshell.org/charcode/

# Variables. {{{1

# Make the code a bit cleaner.
TAB="	"
YES="Yes"
NO="No"

# Defaults.
program="charcode"      # The name of the program being tested.
command="./$program"    # The command to run.
quiet="$NO"             # Quiet mode. (Only failed tests.)
verbose="$NO"           # Verbose output.
onlylist="$NO"          # Only list the sections and names of the tests.

# Internal.
separator="$NO"         # Separator between tests in verbose mode.
fail_count="0"          # The number of test that failed.
pass_count="0"          # The number of test that passed.
prev_section=""         # The previous section.

# These define each test.
# If any variable is set to an empty string it is ignored.
name=""                 # The name of the test. (Required)
description=""          # An option description for the test.
parameters=""           # Command line arguments passed to the program.
exitcode=""             # The expected exit code.
input=""                # Input to pipe to the program.
output=""               # The expected output of the test.

# Functions. {{{1
# usage() - Display the command line help text. {{{2
usage () {
    if [ "$1" ] ; then
        echo "$1"
        echo ""
    fi
    cat<<EOHELP
Usage:  `basename $0` [OPTIONS] [PROGRAM]
A shell utility for testing $program.
Options:
    -h  This help text.
    -l  Only list the sections and names of the tests.
    -q  Quiet output. Print only the names of the tests that fail.
    -v  Verbose output. Overrides quiet.
EOHELP
    exit 1
}

# runtest() - Run a test. {{{2
# $name is the name of the test. $description is an optional extended
# description of the test. The program in $command is run with the arguments
# in $parameters. $input is piped to the STDIN of the program. The output is
# checked against $output and the exit code against $exitcode. For simplicity
# this script is not meant to handle lots of output. Temporary files and cmp
# or diff would work better in that case.
# Variables are used for flexibility and clarity.
runtest () {
    # Deal with the section if there is one and we are not being quiet.
    if [ "$quiet" = "$NO" -a "$section" != "$prev_section" ] ; then
        # Print a separator if this is not the first section..
        if [ ! -z "$prev_section" ] ; then
            echo ""
        fi

        # Print the section.
        echo "Section: $section"
        [ "$separator" = "$NO" ] && echo ""
        prev_section="$section"
    fi

    # Print a separator line between tests in verbose mode.
    if [ "$verbose" = "$YES" ] ; then
        if [ "$separator" = "$YES" ] ; then
            echo ""
        fi
        separator="$YES"
    fi

    # Only print the name if that is what the user wants.
    if [ "$onlylist" = "$YES" ] ; then
        echo "$name"
        return
    fi

    # Default to passed.
    status="Passed"

    # Run the test.
    testout="`echo "$input" | $command $parameters`"
    testexit="$?"

    # Check the exit code.
    if [ "$exitcode" -a "$testexit" -ne "0$exitcode" ] ; then
        status="Failed"
        errexit="$YES"
    else
        errexit="$NO"
    fi

    # Check the output.
    if [ "$output" -a "X$testout" != "X$output" ] ; then
        status="Failed"
        errout="$YES"
    else
        errout="$NO"
    fi

    # Print details if the test failed.
    if [ "$status" != "Passed" ] ; then
        echo "$name ($status)"
        if [ "$quiet" = "$NO" ] ; then
            # Optional description
            if [ "$description" ] ; then
                echo "Description: $description"
            fi

            # Print the command that was executed.
            echo "Command: $command $parameters"

            # Print the input if there was any.
            if [ "$input" ] ; then
                echo "Input ----------------------------------------------------------------"
                echo "$input"
                echo "----------------------------------------------------------------------"
            fi

            # Print the exit code error if it failed.
            if [ "$errexit" = "$YES" ] ; then
                echo "Exit code: $testexit Expected: $exitcode"
            fi

            # Print the output error if it failed.
            if [ "$errout" = "$YES" ] ; then
                echo "Output was:-----------------------------------------------------------"
                echo "$testout"
                echo "Expected:-------------------------------------------------------------"
                echo "$output"
                echo "----------------------------------------------------------------------"
            fi
        fi

        # Add this to the failed count.
        fail_count="`expr $fail_count + 1`"
    elif [ "$quiet" = "$NO" ] ; then
        # Print the name and status.
        echo "$name ($status)"

        # Print the details if being verbose.
        if [ "$verbose" = "$YES" ] ; then
            # Print the command that was executed.
            echo "Command: $command $parameters"

            # Print the input if there was any.
            if [ "$input" ] ; then
                echo "Input ----------------------------------------------------------------"
                echo "$input"
                echo "----------------------------------------------------------------------"
            fi

            # Print the exist code.
            if [ "$exitcode" ] ; then
                echo "Exit Code: $testexit"
            fi

            # Print the output if any was expected.
            if [ "$output" ] ; then
                echo "Output ---------------------------------------------------------------"
                echo "$testout"
                echo "----------------------------------------------------------------------"
            fi
        fi

        # Add this to the passed count.
        pass_count="`expr $pass_count + 1`"
    fi
}

# final_status() - Final status message. {{{2
# Does not print when in quiet mode.
final_status(){
    if [ "$quiet" = "$NO" -a "$onlylist" = "$NO" ] ; then
        total_count="`expr $pass_count + $fail_count`"
        echo ""
        (echo "Total$TAB$total_count"
         echo "Passed$TAB$pass_count"
         echo "Failed$TAB$fail_count") |
         awk -F "$TAB" '{printf "%-6s : %4d\n", $1, $2}'
        if [ "$fail_count" -ne "0" ] ; then
            echo ""
            echo "*** Tests have failed! ***"
            echo "Please email the output of this program (without the -q option) to:"
            echo "LoranceStinson+charcode@gmail.com"
        fi
    fi
}

# Check and handle commandline options. {{{1
while getopts ":hlqv" OPTION ; do
    case $OPTION in
        h)  usage;;                 # Print the help text.
        l)  onlylist=$YES;;         # Only list the sections and names of the tests.
        q)  quiet=$YES;;            # Quite mode.
        v)  verbose=$YES;;          # Verbose output.
        *)  if [ "$OPTARG" = "-" ] ; then
                usage # They probably only want help. Catches --help.
            else
                usage "Invalid option '$OPTARG'." # Illegal option.
            fi;;
    esac
done
shift `expr $OPTIND - 1`

# Verbose overrides quiet.
[ "$verbose" = "$YES" ] && quiet="$NO"

# Is the command specified?
if [ "$#" -ne "0" ] ; then
    command=$1
fi

# Tests. {{{1
# Basic Output Tests. {{{2
section="Basic Output Tests."

# Default output. {{{3
name="Default output."
parameters="Testing!"
exitcode=0
input=""
output='Binary   Oct Dec Hex ASCII
01010100 124 084  54    T
01100101 145 101  65    e
01110011 163 115  73    s
01110100 164 116  74    t
01101001 151 105  69    i
01101110 156 110  6E    n
01100111 147 103  67    g
00100001 041 033  21    !'
runtest

# Output as ASCII. {{{3
name="Output as ASCII."
parameters="-A Testing!"
exitcode=0
input=""
output='Testing!'
runtest

# Output as Binary. {{{3
name="Output as Binary."
parameters="-B Testing!"
exitcode=0
input=""
output='0y01010100 0y01100101 0y01110011 0y01110100 0y01101001 0y01101110 0y01100111 0y00100001'
runtest

# Output as Octal. {{{3
name="Output as Octal."
parameters="-O Testing!"
exitcode=0
input=""
output='124 145 163 164 151 156 147 041'
runtest

# Output as Decimal. {{{3
name="Output as Decimal."
parameters="-D Testing!"
exitcode=0
input=""
output='084 101 115 116 105 110 103 033'
runtest

# Output as Hexadecimal. {{{3
name="Output as Hexadecimal."
parameters="-X Testing!"
exitcode=0
input=""
output='0x54 0x65 0x73 0x74 0x69 0x6E 0x67 0x21'
runtest

# Extended output tests. {{{2
section="Extended output tests."

# Output a single Hexadecimal number. {{{3
name="Output a single Hexadecimal number."
parameters="-Xn Testing!"
exitcode=0
input=""
output='0x54657374696E6721'
runtest

# Output as Hexadecimal by line. {{{3
name="Output as Hexadecimal by line."
parameters="-Xl Testing!"
exitcode=0
input=""
output='0x54
0x65
0x73
0x74
0x69
0x6E
0x67
0x21'
runtest

# Output as Hexadecimal by line with no prefix. {{{3
name="Output as Hexadecimal by line with no prefix."
parameters="-XlP Testing!"
exitcode=0
input=""
output='54
65
73
74
69
6E
67
21'
runtest

# Output as Hexadecimal with no prefix. {{{3
name="Output as Hexadecimal with no prefix."
parameters="-XP Testing!"
exitcode=0
input=""
output='54 65 73 74 69 6E 67 21'
runtest

# Output a single Binary number with no prefix. {{{3
name="Output a single Binary number with no prefix."
parameters="-BnP Testing!"
exitcode=0
input=""
output='0101010001100101011100110111010001101001011011100110011100100001'
runtest

# Output a single Hexadecimal number with no prefix. {{{3
name="Output a single Hexadecimal number with no prefix."
parameters="-XnP Testing!"
exitcode=0
input=""
output='54657374696E6721'
runtest

# Input tests. {{{2
section="Input tests."

# Input as Binary. {{{3
name="Input as Binary."
parameters="-Ab 0y01010100 0y01100101 0y01110011 0y01110100 0y01101001 0y01101110 0y01100111 0y00100001"
exitcode=0
input=""
output='Testing!'
runtest

# Input as Octal. {{{3
name="Input as Octal."
parameters="-Ao 124 145 163 164 151 156 147 041"
exitcode=0
input=""
output='Testing!'
runtest

# Input as Decimal. {{{3
name="Input as Decimal."
parameters="-Ad 084 101 115 116 105 110 103 033"
exitcode=0
input=""
output='Testing!'
runtest

# Input as Hexadecimal. {{{3
name="Input as Hexadecimal."
parameters="-Ax 54 65 73 74 69 6E 67 21"
exitcode=0
input=""
output='Testing!'
runtest

# Extended input tests. {{{2
section="Extended input tests."

# Input a single Binary number. {{{3
name="Input a single Binary number."
parameters="-Ab 0y0101010001100101011100110111010001101001011011100110011100100001"
exitcode=0
input=""
output='Testing!'
runtest

# Input a single Binary number with no prefix. {{{3
name="Input a single Binary number with no prefix."
parameters="-Ab 0101010001100101011100110111010001101001011011100110011100100001"
exitcode=0
input=""
output='Testing!'
runtest

# Input a single Octal number. {{{3
name="Input a single Octal number."
parameters="-Ao 124145163164151156147041"
exitcode=0
input=""
output='Testing!'
runtest

# Input a single Decimal number. {{{3
name="Input a single Decimal number."
parameters="-Ad 084101115116105110103033"
exitcode=0
input=""
output='Testing!'
runtest

# Input a single Hexadecimal number. {{{3
name="Input a single Hexadecimal number."
parameters="-Ax 0x54657374696E6721"
exitcode=0
input=""
output='Testing!'
runtest

# Input a single Hexadecimal number with no prefix. {{{3
name="Input a single Hexadecimal number with no prefix."
parameters="-Ax 54657374696E6721"
exitcode=0
input=""
output='Testing!'
runtest

# Input a single Hexadecimal number from STDIN. {{{3
name="Input a single Hexadecimal number from STDIN."
parameters="-Ax"
exitcode=0
input="0x54657374696E6721"
output='Testing!'
runtest

# Print a final status message. {{{1
final_status
